/* Template 1.1
 * 
 * 04.05.2021
 * Modbus RTU Master für Airzone Aidoo Wifi Controller mit Modbus Schnittstelle
*/

#include <FS.h>                   //this needs to be first, or it all crashes and burns...
#include <ESP8266WiFi.h>          //https://github.com/esp8266/Arduino
#include <AsyncPing.h>
#include <DNSServer.h>
#include <ESP8266WebServer.h>
#include <strings_en.h>
#include <WiFiManager.h>          //https://github.com/tzapu/WiFiManager
#include <ArduinoOTA.h>
#include <ArduinoJson.h>          //https://github.com/bblanchon/ArduinoJson
#include <Ticker.h>;              // für Watchdog
#include <ESP8266mDNS.h>
#include <ESP8266HTTPUpdateServer.h>
#include <SoftwareSerial.h>

WiFiManager wifiManager;
ESP8266WebServer server(80);      // Server auf Port 80 definieren
ESP8266HTTPUpdateServer httpUpdater;
WiFiUDP Udp;

/*Parameter, die im Filesystem persitiert werden.
  Wenn weitere Daten hinzukommen sollen, müssen diese hier und
  in den Routinen "load_config" und "save_config" eingetragen werden.
*/
String        sketchName = "ModbusRtuMaster";       // Name des Sketch
String        sketchVersion = "1.0";                // Version des Sketch
bool          configuredAp = false;                 // gibt an, ob ein Accesspoint für die einwahl konfiguriert ist;
char          host_name[80] = "noname";             // Name des Devices im Netzwerk
char          udp_send_port[6] = "22222";           // Port an den die Daten gesendet werden sollen
int           udp_send_port_i  = 22222;             // Port an den die Daten gesendet werden als Int
char          send_data_to[15] = "192.168.2.3";     // IP Adresse des empfangenden UDP Servers als Text
IPAddress     IP_UDP_Client(192,168,2,3);           // IP Adresse des empfangenden UDP Servers als IP-Adresse

// Daten für AliveSignal
String        aliveConfigFile = "/aliveConfig.json"; // Datei in der die Konfiguartion gespeichert ist
IPAddress     aliveIp(192,168,2,5);                 // IP Adresse an die das AliveSignal gesendet wird
int           alivePort = 8802;                     // Port an die das AliveSignal gesendet wird
int           aliveIntervall = 10000;               // Intervall (ms) in dem das Alive Signal gesendet werden soll
unsigned long nextCheckAlive;                       // enthält die Systemzeit zu der das nächte mal das Alive Signal gesendet werden soll

//für Watchdog
Ticker secondTick;                                  // Zählerobjekt für den Watchdog
volatile int  watchdogCount = 0;                    // Zähler für den Watchdog

//flag for saving data
bool          shouldSaveConfig = false;

#define ONBOARDLED  2                               // GPIO für onboard LED
#define LEDRED      15                              // GPIO für rote LED
#define LEDGREEN    12                              // GPIO für grüne LED
#define LEDBLUE     13                              // GPIO für blaue LED

// für Ping
AsyncPing     PingUdpHost;                          // Objekt zum Pingen
Ticker        pingTimer;                            // Timer um den Ping regelmäßig auszuführen
unsigned long pingNextRebootTime = 120000;          // Zeit zu der neu gestartet wird, wenn keine Antwort auf einen Ping empfangen wird
long          pingTimeout = 600000;                 // Zeit (in ms) nachdem ein Neustart ausgelöst wird, wenn kein Ping Response mehr empfangen wird.
int           pingIntervall = 20;                   // Intervall (in Sekunden) in der gepingt wird.

#define SSerialTxControl 4                          //GPIO for EIA485 Direction control
#define RS485Transmit    HIGH
#define RS485Receive     LOW

void setup() {
  // Serielle Schnittstelle einstellen und EIA485 Modul auf Empfangen stellen
  Serial.begin(19200, SERIAL_8E1);                  // Die Schnittstelle läuft über 19200bps/ 8 bit Datenrahmen/ 1 Stoppbit/ Parität gerade/ keine Datendurchsatzsteuerung
  pinMode(SSerialTxControl, OUTPUT);
  digitalWrite(SSerialTxControl, RS485Receive);     // EIA485 Modul auf Empfangen stellen

  // wenn der ESP8266 noch andere Parameteradaten gespeichert hat, kann es vorkommen,
  // dass er nach dem flashen abstürzt. Dann einmal das Filesystem neu flashen = formatfs();
  // auskommentieren und danach wieder rein nehmen. Dann müssen die Daten neu gesepeichert werden.
  // formatfs();

  // **********************  Laden Daten  **********************
  load_config();        //load configuration from FS json
  loadAliveConfig(aliveIp, alivePort, aliveIntervall, aliveConfigFile);   // laden der Daten AliveSignal
  // ***********************************************************

  // **********************  WiFi konfigurieren  **********************
  WiFi.hostname(host_name);
  // wenn das letzt mal erfolgreich im WLAN eingewählt, dann nur den STA Modus aktivieren. Damit geht dann kein Accesspoint auf.
  if (WiFi.SSID() != "" && getLastWifiLogin() == true) {
    wifiManager.setDebugOutput(false);
    WiFi.mode(WIFI_STA);
  }
  else {
    wifiManager.setDebugOutput(true);
    WiFi.mode(WIFI_AP_STA);
  }
  wifiManager.setSaveConfigCallback(saveConfigCallback);        //set config save notify callback
  wifiManager.setAPCallback(configModeCallback);                //set ...
  wifiManager.setWebServerCallback(webServerCallback);
  wifiManager.setConnectTimeout(30);
  wifiManager.setConfigPortalTimeout(120);
  if (!wifiManager.autoConnect(host_name, "password")) {
    //Serial.println("failed to connect and hit timeout");
    setLastWifiLogin(false);                                    // merken das der Login nicht funtkioniert hat
    delay(1000);
    ESP.reset();
    delay(5000);
  }
  setLastWifiLogin(true);                                       // merken das der Login ok war
  //Serial.println("local ip");
  //Serial.println(WiFi.localIP());
  // ******************************************************************

// melden, per UDP, dass ein Neustart erfolgt ist
   String tempString;
   tempString = "Reboot=";
   tempString += host_name;
   tempString += "; Reason=";
   tempString += ESP.getResetReason();
   tempString += "; ReasonID=";
   rst_info* rinfo = ESP.getResetInfoPtr();
   tempString += rinfo->reason;
   sendUDP(tempString);
   delay(500);                        // Delay ist notwendig, damit sicher per UDP gesendet werden kann.

   // für Ping
   // wird bei jedem Response auf einen Ping aufgerufen
   PingUdpHost.on(true,[](const AsyncPingResponse& response){
     IPAddress addr(response.addr); //to prevent with no const toString() in 2.3.0
     if (response.answer) {
        //String udpDebugText = "Ping Empfang: ";
        //udpDebugText += "%d bytes from %s: icmp_seq=%d ttl=%d time=%d ms\n";
        //udpDebugText += response.size, addr.toString().c_str(), response.icmp_seq, response.ttl, response.time;
        //sendUDP(udpDebugText);
        pingNextRebootTime = millis() + pingTimeout;
     }
     else {
        //String udpDebugText = "no answer yet for icmp_seq=";
        //udpDebugText += addr.toString().c_str(), response.icmp_seq;
        //sendUDP(udpDebugText);
        return false; //do not stop
     }
   });

   // Zusammenfassung nachdem die 3 Pings ausgeführt wurden.
   PingUdpHost.on(false,[](const AsyncPingResponse& response){
     IPAddress addr(response.addr); //to prevent with no const toString() in 2.3.0
     //Serial.printf("total answer from %s sent %d recevied %d time %d ms\n",addr.toString().c_str(),response.total_sent,response.total_recv,response.total_time);
     // sendUDP("Punkt 1");
     if (response.mac)
       // Serial.printf("detected eth address " MACSTR "\n",MAC2STR(response.mac->addr));
       // sendUDP("Punkt 2");
     return true;
   });
   ping();
   pingTimer.attach(pingIntervall,ping);


// --------------------------------------------------------------
// --------- Code für Web-Server --------------------------------
// --------------------------------------------------------------
  server.on ( "/reset", reset );
  server.on ( "/reboot", reboot );
  server.on ( "/formatfs", formatfs );
  server.on ( "/readconfig", read_config );
  server.on ( "/saveconfig", save_config );
  server.on ( "/alivesaveconfig", httpSaveAliveConfig );
  server.on ( "/", help );
  server.on ( "/help", help );
  server.on ( "/set", set );
  server.on ( "/request", request );

  server.onNotFound ( handleNotFound );

  // Start OTA server.
  ArduinoOTA.setHostname(host_name);
  ArduinoOTA.begin();

  MDNS.begin(host_name);
  httpUpdater.setup(&server);
  MDNS.addService("http", "tcp", 80);

  server.begin();


// --------------------------------------------------------------
// --------- Konfiguration der angeschlossenen Geräte -----------
// --------------------------------------------------------------
    

// --------------------------------------------------------------
// --------- Für Watchdog ---------------------------------------
// --------------------------------------------------------------
secondTick.attach(1, ISRwatchdog);

}

void loop() {

  // Der Wachtdogzähler wird hier immer wieder auf 0 gesetzt. Wenn das nicht mehr gemacht wird, dann wird der Watchdog früher oder später ausgelöst
  watchdogCount = 0;

  // Handle OTA server.
  ArduinoOTA.handle();

  server.handleClient();
  yield();
  handleAliveSignal();

  if (WiFi.getMode() == WIFI_AP_STA && millis() > 300000) {
    reboot();
    delay(5000);
  }

  // prüfen, ob ein Neustart erfolgen soll, weil keine Antwort auf den Ping kommt.
  if (millis() > pingNextRebootTime) {ESP.restart();}

}

/*----------------------------------------------------------------------
  ------------- Ab hier kommen Hilfsroutinen aus dem Template ----------
  ----------------------------------------------------------------------*/
//callback notifying us of the need to save config
void saveConfigCallback () {
  //Serial.println("Should save config");
  shouldSaveConfig = true;
}

//callback notifying us of ...
void webServerCallback () {
  //Serial.println("WiFiManager Webserver gestartet");
}

void configModeCallback (WiFiManager *myWiFiManager) {
  //Serial.println("Entered config mode");
  //Serial.println(WiFi.softAPIP());

  //Serial.println(myWiFiManager->getConfigPortalSSID());
}

void reset() {
  String response = "";
  response = "Die WLAN Zugangsdaten werden geloescht und das Geraet neu gestartet.";
  response += "Nach dem Neustart oeffnet das Geraet einen Access-Point ueber den die Konfiguration";
  response += "neu eingestellt werden kann.";
  server.send ( 200, "text/html", response);
  //Serial.println("Reset to Default and Reboot...");
  sendUDP("Reset WLAN Accessdata=1");

  delay(1000);

  WiFiManager wifiManager;
  //reset settings
  wifiManager.resetSettings();

  delay(1000);
  ESP.reset();
}

void reboot() {
  String response = "";
  response = "Softwarerestart erfolgt";
  server.send ( 200, "text/html", response);
  sendUDP("Softwarereboot initiated=1");
  delay(1000);
  ESP.restart();
}

void formatfs() {
  String response = "";
  response = "Filesystem formated.<br>";
  response += "Confing deleted.";
  server.send ( 200, "text/html", response);
  //Serial.println("Filesystem formated. Config deleted...");

  delay(1000);

  //clean FS
  SPIFFS.format();

  delay(1000);
}

void handleNotFound() {
  String message = "File Not Found\n\n";
  message += "URI: ";
  message += server.uri();
  message += "\nMethod: ";
  message += ( server.method() == HTTP_GET ) ? "GET" : "POST";
  message += "\nArguments: ";
  message += server.args();
  message += "\n";

  for ( uint8_t i = 0; i < server.args(); i++ ) {
    message += " " + server.argName ( i ) + ": " + server.arg ( i ) + "\n";
  }

  server.send ( 404, "text/plain", message );
}

// Routine zum senden des übergebenen Textes per UDP
void sendUDP(String Text) {
   int tempLen = Text.length()+1;
   char tempChar[tempLen];
   Text.toCharArray(tempChar,tempLen);
   Udp.beginPacket(IP_UDP_Client, udp_send_port_i);
   Udp.write(tempChar);
   Udp.endPacket();
}

// Routine zum senden des übergebenen Textes per UDP
void sendUDP(String Text, IPAddress ip, int port) {
   int tempLen = Text.length()+1;
   char tempChar[tempLen];
   Text.toCharArray(tempChar,tempLen);
   Udp.beginPacket(ip, port);
   Udp.write(tempChar);
   Udp.endPacket();
}

// sendet bei regelmäßigem Aufruf ein Alive Signal an die konfigurierte Adresse und per UDP
void handleAliveSignal() {
  // prüfen ob das Alive Signal gesendet werden sollen;
  if (millis() > nextCheckAlive) {
    sendUDP("online=1", aliveIp, alivePort);
    nextCheckAlive = millis() + aliveIntervall;
  }
}

// Watchdog Routine, die prüft, ob ein Reset ausgelöst werden muss
void ISRwatchdog() {
  watchdogCount++;
  if (watchdogCount == 60) {
    //Serial.println("wachtdog reset......");
    ESP.reset();
  }
}

void ping(){
    // Serial.printf("started ping to %s:\n",IP_UDP_Client.toString().c_str());
    PingUdpHost.begin(IP_UDP_Client);
}

